"""names.py

Utility functions for serializing and deserializing Antelope names.

Based off eosjs
"""


def char_to_symbol(c: int) -> int:
    """utility encoder from character int to symbol int"""
    if ord("a") <= c <= ord("z"):
        return c - ord("a") + 6
    if ord("1") <= c <= ord("5"):
        return c - ord("1") + 1
    return 0


def symbol_to_char(s: int) -> str:
    """Utility encoder from symbol int to str"""
    if 6 <= s <= 31:
        return chr(ord("a") + s - 6)
    if 1 <= s <= 5:
        return chr(ord("1") + s - 1)
    return "."


def serialize_name(s: str) -> bytes:
    """Converts the string representation to an 8 byte Antelope name"""
    a = bytearray(8)
    bit = 63
    for i in s:
        c = char_to_symbol(ord(i))
        if bit < 5:
            c = c << 1
        for j in range(4, -1, -1):
            if bit >= 0:
                a[bit // 8] |= ((c >> j) & 1) << (bit % 8)
                bit -= 1
    return a


def deserialize_name(v: bytes):
    """Converts an Antelope name (8 bytes) to string"""
    result = ""
    bit = 63
    while bit >= 0:
        c = 0
        for _ in range(0, 5):
            if bit >= 0:
                c = (c << 1) | ((v[bit // 8] >> bit % 8) & 1)
                bit -= 1
        result += symbol_to_char(c)
    result = result.rstrip(".")
    return result



""" varints.py

Varint Implementation based off https://github.com/fmoo/python-varint
"""
from typing import Tuple


def serialize_varint(n: int) -> bytes:
    """Encodes a positive integer as a varint

    Args:
        n (int): the integer to encode

    Returns:
        bytes: encoded int
    """
    buf = b""
    while True:
        chunk = n & 0x7F
        n >>= 7
        if n:
            buf += (chunk | 0x80).to_bytes(1, "little")
        else:
            buf += chunk.to_bytes(1, "little")
            break
    return buf


# Stream conversion isn't needed for this project, and iterating
# over the buffer to decode the varint is faster
def deserialize_varint(n: bytes) -> Tuple[int, bytes]:
    """Decodes a varint from bytes"""
    result = 0
    pos = 0
    reader = iter(n)
    for byte in reader:
        result += (byte & 0x7F) << pos
        if not byte & 0x80:
            break
        pos += 7
    return result, bytes(reader)



"""Extra type hinting types"""
from typing import Literal

# The default types for Antelope, and the `struct` module format string where relevant
DEFAULT_TYPES = {
    "bool": "",
    "int8": "b",
    "uint8": "B",
    "int16": "h",
    "uint16": "H",
    "int32": "i",
    "uint32": "I",
    "int64": "q",
    "uint64": "Q",
    "int128": "",  # TODO Struct doesn't natively 128bit support, so split before encoding
    "uint128": "",  # TODO Struct doesn't natively 128bit support, so split before encoding
    "varint32": "",
    "varuint32": "",
    "float32": "f",
    "float64": "d",
    "float128": "",  # TODO Struct doesn't natively 128bit support, so split before encoding
    "time_point": "",
    "time_point_sec": "",
    "block_timestamp_type": "",  # TODO
    "name": "",
    "bytes": "",
    "string": "",
    "checksum160": "",
    "checksum256": "",
    "checksum512": "",
    "public_key": "",
    "signature": "",
    "symbol": "",
    "symbol_code": "",
    "asset": "",
    "extended_asset": "",  # TODO
}


ValidTypes = Literal[
    "bool",
    "int8",
    "uint8",
    "int16",
    "uint16",
    "int32",
    "uint32",
    "int64",
    "uint64",
    "int128",
    "uint128",
    "varint32",
    "varuint32",
    "float32",
    "float64",
    "float128",
    "time_point",
    "time_point_sec",
    "block_timestamp_type",
    "name",
    "bytes",
    "string",
    "checksum160",
    "checksum256",
    "checksum512",
    "public_key",
    "signature",
    "symbol",
    "symbol_code",
    "asset",
    "extended_asset",
]



import struct
from typing import Any, Protocol, Tuple, Union

from antelopy.serializers import names, varints
from antelopy.types.types import DEFAULT_TYPES


class Serializer(Protocol):
    """Base Serializer Protocol"""

    def serialize(self, v: Any) -> Any:
        ...

    def deserialize(self, v: Any) -> Any:
        ...


class NameSerializer(Serializer):
    def serialize(self, v: str) -> bytes:
        return names.serialize_name(v)

    def deserialize(self, v: bytes) -> str:
        return names.deserialize_name(v)


class NumberSerializer(Serializer):
    def __init__(self, type: str):
        self.type = type

    def serialize(self, n: Union[int, float]) -> bytes:
        return struct.pack(DEFAULT_TYPES[self.type], n)

    def deserialize(self, v: bytes) -> str:
        ...


class StringSerializer(Serializer):
    def serialize(self, v: str) -> bytes:
        return VaruintSerializer().serialize(len(v)) + v.encode("utf-8")

    def deserialize(self, v: Any) -> Any:
        ...


class VarintSerializer(Serializer):
    def serialize(self, v: int) -> bytes:
        return varints.serialize_varint((v << 1) ^ (v >> 31))

    def deserialize(self, v: bytes) -> Tuple[int, bytes]:
        return varints.deserialize_varint(v)


class VaruintSerializer(Serializer):
    def serialize(self, v: int) -> bytes:
        return varints.serialize_varint(v)

    def deserialize(self, v: bytes) -> Tuple[int, bytes]:
        return varints.deserialize_varint(v)



"""exceptions.py

Exceptions to help flag errors with serialization"""


class ABINotFoundError(Exception):
    """No ABI found for account"""


class ABINotCachedError(Exception):
    """ABI hasn't been cached yet"""


class AccountNotFoundError(Exception):
    """This account couldn't be found"""


class ActionNotFoundError(Exception):
    """Action not found in ABI"""


class ActionMissingFieldError(Exception):
    """Missing field in action serialization"""



"""base58.py

Implementation of base58 encoding because no version of base58 is compatible
with aioeos, eospy, and pyntelope :P
"""
from typing import Union

ALPHABET = "123456789ABCDEFGHJKLMNPQRSTUVWXYZabcdefghijkmnopqrstuvwxyz"


def b58encode(b: Union[bytes, str]) -> bytes:
    """Converts string or bytes to Base58 encoded bytes

    Args:
        b (Union[bytes, str]): input value

    Returns:
        bytes: b58 encoded bytes
    """
    if isinstance(b, str):
        b = b.encode("ascii")
    r = b""
    zeros = 0
    for i in b:
        if i == 0:
            zeros += 1
        else:
            break
    n = int.from_bytes(b, "big")
    while n:
        n, mod = divmod(n, 58)
        r = ALPHABET[mod].encode() + r
    return b"1" * zeros + r


def b58decode(b: Union[bytes, str]) -> bytes:
    """Converts b58 encoded bytes or str to ascii bytes

    Args:
        b (Union[bytes, str): b58 encoded input

    Returns:
        bytes: bytes
    """
    if isinstance(b, str):
        b = b.encode("ascii")
    ones = 0
    for i in b:
        if chr(i) == "1":
            ones += 1
        else:
            break
    r = 0
    p = 1
    for i in b:
        r += (58 ** (len(b) - p)) * ALPHABET.index(chr(i))
        p += 1
    return b"\x00" * ones + r.to_bytes((r.bit_length() + 7) // 8, "big")



"""assets.py

Utility functions for assets and symbols"""

import struct
from decimal import Decimal


def serialize_symbol_code(symbol_name: str) -> bytes:
    """Converts a precision and symbol (token name) to bytes for serialization

    Args:
        precision (int): precision of the token
        symbol_name (str): token name (max 7 characters)

    Returns:
        bytes: serialized data
    """
    symbol_bytes = symbol_name.encode()
    while len(symbol_bytes) < 7:
        symbol_bytes += b"\x00"
    return symbol_bytes


# Symbol consists of precision,symbol_name uint8, string
# String gets padded to uint64
def serialize_symbol(precision: int, symbol_name: str) -> bytes:
    """Converts a precision and symbol (token name) to bytes for serialization

    Args:
        precision (int): precision of the token
        symbol_name (str): token name (max 7 characters)

    Returns:
        bytes: serialized data
    """
    precision_byte = struct.pack("B", precision)
    return precision_byte + serialize_symbol_code(symbol_name)


# Formated as 1.000000 WAX
def serialize_asset(asset_string: str) -> bytes:
    """Converts an asset string to bytes for serialization

    Args:
        asset_string (str): string to encode, e.g. "1.23450000 WAX"

    Returns:
        bytes: serialized data
    """
    quantity, symbol_name = asset_string.split(" ")
    if "." not in quantity:
        precision = 0
    else:
        precision = len(quantity.split(".")[1])
    amount = int(Decimal(quantity) * 10**precision)
    amount_bytes = struct.pack("Q", amount)
    symbol_bytes = serialize_symbol(precision, symbol_name)
    return amount_bytes + symbol_bytes



"""keys.py

Utility functions to serialize EOS keys and signatures into bytes"""

import struct
from datetime import datetime
from typing import Union


def serialize_time_point(value: Union[int, float, datetime]) -> bytes:
    """Converts a datetime or datetime.timestamp float to serialized bytes with millisecond
    precision

    Args:
        timestamp (float | datetime): ts
    """
    if isinstance(value, int):
        pass
    elif isinstance(value, float):
        value = int(value * 1000)
    elif isinstance(value, datetime):
        value = int(value.timestamp() * 1000)
    else:
        raise TypeError(f"Expected int, float, or datetime. Got {type(value)}")
    return struct.pack("Q", value)


def serialize_time_point_sec(value: Union[int, float, datetime]) -> bytes:
    """Converts a datetime or datetime.timestamp float to serialized bytes with second precision

    Args:
        timestamp (float | datetime): ts
    """
    if isinstance(value, int):
        pass
    elif isinstance(value, float):
        value = int(value)
    elif isinstance(value, datetime):
        value = int(value.timestamp())
    else:
        raise TypeError(f"Expected int, float, or datetime. Got {type(value)}")
    return struct.pack("I", value)



from typing import Any, Dict, List, Protocol

from antelopy.types import serializers


class Serializable(Protocol):
    def serialize(self):
        ...

    def deserialize(self):
        ...


class BasicSerializer(Serializable):
    def __init__(self, value: Any, serialization_strategy: serializers.Serializer):
        self.value = value
        self.strategy = serialization_strategy

    def serialize(self):
        return self.strategy.serialize(self.value)

    def deserialize(self):
        return self.strategy.deserialize(self.value)


class ListSerializable(Serializable):
    def __init__(
        self, values: List[Serializable], serialization_strategy: serializers.Serializer
    ):
        self.values = values
        self.strategy = serialization_strategy

    def serialize(self):
        serialized_values = [v.serialize() for v in self.values]
        return self.strategy.serialize(serialized_values)

    def deserialize(self):
        return self.strategy.deserialize(self.values)


class DictSerializable(Serializable):
    def __init__(
        self,
        values: Dict[str, Serializable],
        serialization_strategy: serializers.Serializer,
    ):
        self.values = values
        self.strategy = serialization_strategy

    def serialize(self):
        return self.strategy.serialize(self.values)

    def deserialize(self):
        return self.strategy.deserialize(self.values)


SERIALIZER_MAP = {
    "name": serializers.NameSerializer(),
    "string": serializers.StringSerializer(),
    "uint8": serializers.NumberSerializer("uint8"),
    "int8": serializers.NumberSerializer("int8"),
}



"""abicache.py

Core class of abicache package"""

import binascii
import hashlib
import json
from typing import Any, Dict, Union

from antelopy.cache.chain_interface import ChainInterface
from antelopy.exceptions.exceptions import ABINotCachedError, ActionNotFoundError
from antelopy.types.abi import Abi


class AbiCache:
    """Cache for imported ABIs

    ...

    Attributes
    ----------
    chain : str
        Antelope chain endpoint to use
    abi_cache : dict[str, Abi]
        internal cache of Abi objects, indexed by account name.

    Methods
    -------
    info(additional=""):
        Prints the person's name and age.
    """

    def __init__(self, chain_endpoint: str):
        self.chain = ChainInterface(chain_endpoint)
        self._abi_cache: Dict[str, Abi] = {}

    def dump_abi(self, account_name: str, path: str) -> None:
        """Dumps an ABI of an account into path

        Args:
            account_name (str): account name
        """
        raw_abi = self.chain.get_raw_abi(account_name)
        with open(path, "w+", encoding="utf-8") as jfp:
            json.dump(raw_abi, jfp, indent=2)

    def read_abi(self, account_name: str) -> None:
        """Loads an ABI of an account into memory

        Args:
            account_name (str): account name
        """
        raw_abi = self.chain.get_raw_abi(account_name)
        self._abi_cache[account_name] = Abi(name=account_name, **raw_abi)

    def read_abi_from_json(self, account_name: str, path: str) -> None:
        """Loads an ABI of an account into memory

        Args:
            account_name (str): account name
        """
        with open(path, "r", encoding="utf-8") as jfp:
            abi = json.load(jfp)
        self._abi_cache[account_name] = Abi(name=account_name, **abi)

    def serialize_data(
        self, contract_name: str, contract_action: str, data: Dict[str, Any]
    ) -> bytes:
        """Serializes an action into a hex-encoded bytestring

        Args:
            contract_name (str): smart contract name
            contract_action (str): smart contract action
            data (Dict[str, Any]): action data

        Raises:
            Exception: _description_

        Returns:
            bytes: _description_
        """
        abi = self._abi_cache.get(contract_name)
        if not abi:
            raise ABINotCachedError(
                f"ABI {contract_name} hasn't been cached yet. Use read_abi or read_abi_from_json"
            )
        action = abi.get_action(contract_action)
        if not action:
            raise ActionNotFoundError(
                f"Action {contract_action} not found in ABI for {contract_name}"
            )
        return binascii.hexlify(abi.serialize(action, data))

    def hexlify(self, data: bytes) -> bytes:
        """Wrapper around binascii.hexlify to avoid extra imports"""
        return binascii.hexlify(data)

    def unhexlify(self, data: Union[bytes, str]) -> bytes:
        """Wrapper around binascii.unhexlify to avoid extra imports"""
        return binascii.unhexlify(data)

    def sha256digest(self, string: bytes) -> bytes:
        """Wrapper around hashlib.sha256().digest()"""
        return hashlib.sha256(string).digest()



"""keys.py

Utility functions to serialize EOS keys and signatures into bytes"""

import struct

from antelopy.utils.base58 import b58decode

KEY_TYPES = {
    "k1": 0,
    "r1": 1,
    "wa": 2,
}
# TODO: add reverse implementations, and potentially key length validation
# based on eosjs-numeric.ts (EOSIO/eosjs)


def serialize_public_key(s: str):
    """Converts string key (EOS...) to bytes with leading key-type byte"""
    buf = b""
    if s[:3] == "EOS":
        key_type = KEY_TYPES["k1"]
        buf += struct.pack("b", key_type)
        buf += b58decode(s[3:])[:-4]
    elif s[:3] == "PUB":
        key_type = KEY_TYPES[s[4:6].lower()]
        sig = s[7:]
        buf = b""
        buf += struct.pack("b", key_type)
        buf += b58decode(sig)[:-4]
    return buf


def serialize_signature(s: str):
    """Converts string signature (SIG_XX_...) to bytes with leading key-type byte"""
    key_type = KEY_TYPES[s[4:6].lower()]
    sig = s[7:]
    buf = b""
    buf += struct.pack("b", key_type)
    buf += b58decode(sig)[:-4]
    return buf



from typing import Any, Dict

import requests

from antelopy.exceptions.exceptions import ABINotFoundError, AccountNotFoundError


class ChainInterface:
    def __init__(self, chain_endpoint: str):
        self.session = requests.Session()
        self.endpoint = chain_endpoint

    def get_raw_abi(self, account_name: str) -> Dict[str, Any]:
        """Reads a raw ABI file from chain

        Args:
            account_name (str): name of account with ABI

        Raises:
            AccountNotFoundError: Account not found on chain
            Exception: Panic #
            ABINotFoundError: Account has no ABI

        Returns:
            Dict[str,Any]: _description_
        """
        r = self.session.post(
            f"{self.endpoint}/v1/chain/get_abi", json={"account_name": account_name}
        )
        s = r.status_code
        if s == 500:
            # TODO: Add specific errors
            raise AccountNotFoundError(
                f"Couldn't find account {account_name}. Error JSON" + "\n" + r.json()
            )
        if s == 200:
            pass
        else:
            raise Exception("Couldn't get data from chain")
        if raw_abi := r.json().get("abi"):
            return raw_abi
        raise ABINotFoundError(f"Couldn't retrieve ABI for {account_name}")



"""abi.py
Contains the Abi type classes for ABI interactions"""
import binascii
import struct
from typing import Any, List, Union, cast

from pydantic import BaseModel

from antelopy.exceptions.exceptions import ActionMissingFieldError
from antelopy.serializers import assets, keys, names, time_points, varints
from antelopy.types.types import DEFAULT_TYPES, ValidTypes


class AbiBaseClass(BaseModel):
    """Inherited subclass for easy data type checks"""

    type: str = ""
    is_list: bool = False

    def __str__(self):
        return self.type


class AbiType(AbiBaseClass):
    """Types that extend to the standard Antelope types."""

    new_type_name: str

    def __init__(self, **data: Any):
        super().__init__(**data)
        if self.type.endswith("[]"):
            self.is_list = True
            self.type = self.type[:-2]


class AbiStructField(AbiBaseClass):
    name: str
    optional: bool = True

    def __init__(self, **data: Any):
        super().__init__(**data)
        if self.type.endswith("[]"):
            self.is_list = True
            self.type = self.type[:-2]
        if self.type.endswith("$"):
            self.optional = True

    def __str__(self):
        return self.type


class AbiStruct(AbiBaseClass):
    name: str
    base: str
    fields: List[AbiStructField]


class AbiAction(AbiBaseClass):
    name: str
    type: str
    ricardian_contract: str
    fields: List[AbiStructField] = []


class AbiTables(AbiBaseClass):
    ...


class AbiRicardianClauses(AbiBaseClass):
    ...


class AbiErrorMessages(AbiBaseClass):
    ...


class AbiExtensions(AbiBaseClass):
    ...


class AbiVariants(AbiBaseClass):
    name: str
    types: List[str]


class Abi(AbiBaseClass):
    name: str = ""
    version: str = ""
    types: List[AbiType] = []
    structs: List[AbiStruct] = []
    actions: List[AbiAction] = []
    tables: List[AbiTables] = []
    ricardian_clauses: List[AbiRicardianClauses] = []
    error_messages: List[AbiErrorMessages] = []
    abi_extensions: List[AbiExtensions] = []
    variants: List[AbiVariants] = []
    # action_results: list = []

    def __init__(self, name: str, **data: Any):
        super().__init__(**data)
        self.name = name
        for action in self.actions:
            for s in self.structs:
                if action.name == s.name:
                    action.fields = s.fields
                    break

    def get_action(self, action_name: str):
        actions = [a for a in self.actions if a.name == action_name]
        if actions:
            return actions[0]
        return None

    def resolve_data_type(self, field: Union[AbiType, AbiStructField, str]):
        # if field.type in DEFAULT_TYPES:
        #     return field.type
        field_type = str(field)
        if type_options := [t for t in self.types if field_type == t.new_type_name]:
            return type_options[0]
        if struct_options := [s for s in self.structs if field_type == s.name]:
            return struct_options[0]
        if variant_options := [v for v in self.variants if field_type == v.name]:
            return variant_options[0]

        raise Exception

    def serialize_default(self, t: ValidTypes, value: Any) -> bytes:
        """Serializes default Antelope types

        Args:
            t (str): type name
            value (_type_): the basic value to be serialized

        Returns:
            bytes: _description_
        """
        buf = b""
        if t == "name":
            buf += names.serialize_name(value)
        elif t in (
            "uint8",
            "uint16",
            "uint32",
            "uint64",
            "int8",
            "int16",
            "int32",
            "int64",
        ):
            if not isinstance(value, int):
                value = int(value)
            # if value < 0:
            #     bit_length = int(t.split("int")[1])
            #     value = (1 << bit_length) + value  # quick two's compliment
            buf += struct.pack(DEFAULT_TYPES[t], value)
        elif t in ("float32", "float64"):
            if not isinstance(value, float):
                value = float(value)
            buf += struct.pack(DEFAULT_TYPES[t], value)
        elif t == "string":
            if not isinstance(value, str):
                raise ValueError(
                    f"Expected a string for this field, got {type(value)} instead"
                )
            buf += varints.serialize_varint(len(value)) + value.encode("utf-8")
        elif t == "bytes":
            if not isinstance(value, bytes):
                raise ValueError(
                    f"Expected a bytes object for this field, got {type(value)} instead"
                )
            buf += varints.serialize_varint(len(value)) + value
        elif t == "bool":
            buf += b"\x01" if value else b"\x00"
        elif t == "public_key":
            buf += keys.serialize_public_key(value)
        elif t == "signature":
            buf += keys.serialize_signature(value)
        elif t == "symbol_code":
            buf += assets.serialize_symbol_code(value)
        elif t == "symbol":
            precision, symbol_code = value.split(",")
            buf += assets.serialize_symbol(int(precision), symbol_code)
        elif t == "asset":
            buf += assets.serialize_asset(value)
        elif t == "time_point":
            buf += time_points.serialize_time_point(value)
        elif t == "time_point_sec":
            buf += time_points.serialize_time_point_sec(value)
        elif t in ["varuint32", "varint32"]:
            if t == "varint32":
                value = (value << 1) ^ (value >> 31)  # 32-1
            buf += varints.serialize_varint(value)
        elif t.startswith("checksum"):
            if isinstance(value, str):
                buf += bytes.fromhex(value)
            elif isinstance(value, bytes):
                try:
                    # test if hex-encoded bytes
                    value = binascii.unhexlify(value)
                except binascii.Error:
                    ...
                if len(value) in [20, 32, 64]:
                    buf += value
            else:
                raise ValueError("serializing checksums expects str or bytes format")
        else:
            raise Exception(f"Type {t} isn't handled yet")
        return buf

    def serialize_list(
        self, t: Union[AbiType, AbiStructField], value: List[Any]
    ) -> bytes:
        buf = b""
        buf += varints.serialize_varint(len(value))
        for i in value:
            if t.type in DEFAULT_TYPES:
                buf += self.serialize_default(cast(ValidTypes, t.type), i)
            else:
                new_type = self.resolve_data_type(t.type)
                if isinstance(new_type, AbiStruct):
                    buf += self.serialize(new_type, i)
                else:
                    buf += self.serialize_non_default(new_type, i)

        return buf

    def serialize_variant(self, variant_types: List[str], value: Any):
        buf = b""

        value_type, v = value
        buf += varints.serialize_varint(variant_types.index(value_type))
        if value_type in DEFAULT_TYPES:
            buf += self.serialize_default(value_type, v)
        else:
            new_type = self.resolve_data_type(value_type)
            if isinstance(new_type, AbiStruct):
                buf += self.serialize(new_type, v)
            else:
                buf += self.serialize_non_default(new_type, v)
        return buf

    def serialize_non_default(
        self, t: Union[AbiType, AbiStructField, AbiVariants], value: Any
    ) -> bytes:
        buf = b""
        # handle types which are just renamed default types
        if t.is_list:
            buf += self.serialize_list(cast(Union[AbiType, AbiStructField], t), value)
            return buf
        if isinstance(t.type, str) and t.type in DEFAULT_TYPES:
            buf += self.serialize_default(cast(ValidTypes, t.type), value)
            return buf
        if type_options := [nt for nt in self.types if t.type == nt.new_type_name]:
            buf += self.serialize_non_default(type_options[0], value)
        elif struct_options := [s for s in self.structs if t.type == s.name]:
            s = struct_options[0]
            buf += self.serialize(s, value)
        elif variant_options := [v for v in self.variants if t.type == v.name]:
            variant = variant_options[0]
            buf += self.serialize_variant(variant.types, value)
        return buf

    def serialize(self, action: Union[AbiAction, AbiStruct], data: Any) -> bytes:
        buf = b""
        for field in action.fields:
            value = data.get(field.name)
            if value is None:
                if field.optional:
                    continue
                raise ActionMissingFieldError(
                    f"Action {action.name} is missing field {field.name}"
                )

            if field.type in DEFAULT_TYPES and not field.is_list:
                buf += self.serialize_default(cast(ValidTypes, field.type), value)
                continue
            buf += self.serialize_non_default(field, value)
        return buf



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['antelopy',
 'antelopy.cache',
 'antelopy.exceptions',
 'antelopy.serializers',
 'antelopy.types',
 'antelopy.utils']

package_data = \
{'': ['*']}

install_requires = \
['pydantic>=2.4.2,<3.0.0', 'requests>=2.31.0,<3.0.0']

setup_kwargs = {
    'name': 'antelopy',
    'version': '0.1.6',
    'description': 'Python helper for Antelope transaction serialization',
    'long_description': '# antelopy\n\n![Workflow Badge](https://github.com/stuckatsixpm/antelopy/actions/workflows/main.yml/badge.svg?branch=main) ![Python version](https://img.shields.io/badge/python-3.8%20%7C%203.9%20%7C%203.10%20%7C%203.11%20%7C%203.12-blue) ![PyPI](https://img.shields.io/pypi/v/antelopy?label=PyPI)\n\n*v0.1.6 - initial release*\n\nDrop-in Python ABI cache for Antelope chains with local serialization support. \n\n## Features\n* Serialization of Antelope built-in types\n* Caches ABIs for faster serialization of actions\n* Support for ABI custom types/variants\n\n## Basic Usage:\n*Note: Reading ABIs uses `requests` and is not asynchronous.*\n\n### Instaliation\n\n```bash\npip install antelopy\n```\n\n### Example with aioeos\n**Loading a contract\'s ABI into the cache:**\n```py\nfrom antelopy import AbiCache\n\nCHAIN_ENDPOINT = "https://wax.eosphere.io"\n\n# Create ABI Cache and read the Atomic Assets contract ABI\nabicache = AbiCache(chain_endpoint=CHAIN_ENDPOINT)\nabicache.read_abi("atomicassets")\n```\n\n\n**Serializing, signing, and pushing a transaction** *(modified version of aioeos\' built-in `EosTransaction.sign_and_push_transaction` function)*\n```py\nimport asyncio\nfrom antelopy import AbiCache\nfrom aioeos import EosAccount, EosJsonRpc, EosTransaction, serializer\n\nCHAIN_ENDPOINT = "https://wax.eosphere.io"\n\n# Create ABI Cache and read the Atomic Assets contract ABI\nabicache = AbiCache(chain_endpoint=CHAIN_ENDPOINT)\nabicache.read_abi("atomicassets")\n\n# Fake Account\nwax_account = EosAccount(\n    name="testaccount1",\n    private_key="your private key"\n)\n\n# \nrpc = EosJsonRpc(CHAIN_ENDPOINT)\n\ntransaction = EosTransaction(\n    # transaction data\n)\nasync def serialize_sign_and_push(transaction: EosTransaction):\n    for action in transaction.actions: \n        if isinstance(action.data, dict):\n            # This {"binargs": serialized_data} structure emulates\n            # the response from the old `abi_json_to_bin` endpoint.\n            abi_bin = {"binargs":abicache.serialize_data(action.account,action.name, action.data)}\n            action.data = abi_cache.unhexlify(abi_bin[\'binargs\'])\n\n    chain_id = await RPC.get_chain_id()\n    serialized_transaction = serializer.serialize(transaction)\n\n    digest = abi_cache.sha256digest(\n        b\'\'.join((chain_id, serialized_transaction, bytes(32)))\n    )\n\n    return await RPC.push_transaction(\n        signatures=[key.sign(digest) for key in [wax_account.key]],\n        serialized_transaction=(\n            abi_cache.hexlify(serialized_transaction).decode()\n        )\n    )\n\nawait serialize_sign_and_push(transaction)\n```\n\n## Todo:\n* Implement remaining types\n* refactor serializers to class based approach, similar to [aioeos](https://github.com/ulamlabs/aioeos/blob/master/aioeos/serializer.py)\n* Implement better type hinting for serialization\n* Expand test coverage\n* Add examples for aioeos, eospy, and pyantelope',
    'author': 'Jake Hattwell',
    'author_email': 'stuck@sixpm.dev',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/stuckatsixpm/antelopy',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.8,<4.0',
}


setup(**setup_kwargs)



"""Antelopy

Python package to handle Antelope action data serialization

See https://github.com/stuckatsixpm/antelopy for examples."""
from antelopy.cache.abicache import AbiCache

__all__ = ["AbiCache"]
